package pay.platform.core.security.config;

import pay.platform.core.security.expression.PermissionService;
import pay.platform.core.security.filter.JwtTokenAuthenticationFilter;
import pay.platform.core.security.handler.TokenAccessDeniedHandler;
import pay.platform.core.security.handler.TokenAuthenticationEntryPoint;
import pay.platform.core.util.PasswordUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.annotation.web.configurers.HeadersConfigurer;
import org.springframework.security.config.annotation.web.configurers.HttpBasicConfigurer;
import org.springframework.security.config.core.GrantedAuthorityDefaults;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.filter.CorsFilter;

/**
 * Spring Security 配置
 */
@Configuration
// web 支持
@EnableWebSecurity
// 对注解
//  @PreAuthorize, @PostAuthorize, @PreFilte
@EnableMethodSecurity
@RequiredArgsConstructor
public class WebSecurityConfiguration {

    private final UserDetailsService userDetailsService;

    private final CorsFilter corsFilter;

    private final JwtTokenAuthenticationFilter tokenAuthenticationFilter;

    private final TokenAccessDeniedHandler tokenAccessDeniedHandler;

    private final TokenAuthenticationEntryPoint authenticationEntryPoint;

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                // csrf 禁用
                .csrf(AbstractHttpConfigurer::disable)
                // 异常处理
                .exceptionHandling((exceptionHandling) -> {
                    exceptionHandling.authenticationEntryPoint(authenticationEntryPoint).accessDeniedHandler(tokenAccessDeniedHandler);
                }).userDetailsService(userDetailsService)
                // 防止iframe 造成跨域
                .headers((headers) -> headers.frameOptions(HeadersConfigurer.FrameOptionsConfig::disable))
                // session 禁用,采用token方式
                .sessionManagement((sessionManagement) -> sessionManagement.sessionCreationPolicy(SessionCreationPolicy.STATELESS)).formLogin(AbstractHttpConfigurer::disable).httpBasic(HttpBasicConfigurer::disable)

                .authorizeHttpRequests((authorizeHttpRequests) -> authorizeHttpRequests
                        // swagger 文档
                        .requestMatchers("/swagger-ui.html").permitAll().requestMatchers("/swagger-resources/**").permitAll().requestMatchers("/webjars/**").permitAll().requestMatchers("/*/api-docs/**").permitAll()
                        // 静态资源等等
                        .requestMatchers(HttpMethod.GET, "/*.html", "/*/*.html", "/*/*.css", "/*/*.js", "/webSocket/**").permitAll()
                        //对于登录login / 获取用户IP / OpenAPI  开放
                        .requestMatchers("/auth/**", "/ip/**", "/api/**", "/error", "/favicon.ico").permitAll()
                        // options 开放
                        .requestMatchers(HttpMethod.OPTIONS, "/**").permitAll()
                        // 文件
                        .requestMatchers("/avatar/**").permitAll().requestMatchers("/file/**").permitAll()
                        // 其余认证访问
                        .anyRequest().authenticated())
                // filter
                .addFilterBefore(corsFilter, CorsFilter.class).addFilterBefore(tokenAuthenticationFilter, UsernamePasswordAuthenticationFilter.class);
        return http.build();
    }


    @Bean
    public AuthenticationManager authenticationManager(AuthenticationConfiguration authenticationConfiguration) throws Exception {
        return authenticationConfiguration.getAuthenticationManager();
    }


    @Bean
    public PasswordEncoder passwordEncoder() {
        return PasswordUtil.defaultPasswordEncoder();
    }

    /**
     * 移除{@code  ROLE_}前缀,
     * 方便使用{@link  PermissionService#hashPermission(String)}或者Spring
     * Security自己的方法
     *
     * @return .
     */
    @Bean
    public GrantedAuthorityDefaults grantedAuthorityDefaults() {
        return new GrantedAuthorityDefaults("");
    }
}
