package pay.platform.api.system.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import pay.platform.api.system.cache.RouteCache;
import pay.platform.api.system.model.dto.PermissionTree;
import pay.platform.api.system.model.query.PermissionTreeQuery;
import pay.platform.api.system.model.vo.VueMenuRouteVO;
import pay.platform.api.system.servcie.SysPermissionService;
import pay.platform.core.common.Result;
import pay.platform.core.security.model.UserInfo;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.SysPermission;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * 系统：菜单与权限
 */
@RestController
@RequestMapping("/sys/permission")
@Tag(name = "系统：菜单与权限")
@RequiredArgsConstructor
public class SysPermissionController {

    private final SysPermissionService sysPermissionService;

    private final RouteCache routeCache;

    /**
     * 获取当前用户路由
     *
     * @param request .
     * @return .
     */
    @GetMapping("/routes")
    @Operation(summary = "获取当前用户路由")
    public Result<List<VueMenuRouteVO>> getCurrentRoute(HttpServletRequest request) {
        UserInfo user = SecurityUtil.getCurrentUser();
        Optional<List<VueMenuRouteVO>> routeCacheOptional = routeCache.getCache(user.getUserid());
        if (routeCacheOptional.isPresent()) {
            return Result.OK(routeCacheOptional.get());
        }
        List<VueMenuRouteVO> menuRoute = sysPermissionService.queryRouteByUserid(user.getUserid());

        routeCache.putCache(user.getUserid(), menuRoute);
        return Result.OK(menuRoute);
    }

    /**
     * 获取菜单与权限
     *
     * @return .
     */
    @GetMapping("/tree")
    @Operation(summary = "获取菜单与权限树形列表")
    @PreAuthorize("@permission.hashPermission('menu:query','role:query')")
    public Result<List<PermissionTree>> treesPermission(@ParameterObject PermissionTreeQuery query) {
        Optional<List<SysPermission>> permissionOpt = sysPermissionService.listPermission(query);
        Optional<List<PermissionTree>> treeOpt = permissionOpt
                .flatMap(e -> Optional.ofNullable(sysPermissionService.buildTree(e)));
        List<PermissionTree> tree = treeOpt.orElse(Collections.emptyList());
        return Result.OK(tree);
    }

    @GetMapping("/tree/menu")
    @Operation(summary = "获取菜单树形列表")
    public Result<List<PermissionTree>> tressMenus(@ParameterObject PermissionTreeQuery query) {
        Optional<List<SysPermission>> permissionOpt = sysPermissionService.listMenu(query);
        Optional<List<PermissionTree>> treeOpt = permissionOpt
                .flatMap(e -> Optional.ofNullable(sysPermissionService.buildTree(e)));
        List<PermissionTree> tree = treeOpt.orElse(Collections.emptyList());
        return Result.OK(tree);
    }

}
